;;;; -*- lisp -*-

(in-package :it.bese.tar)

(defun write-tar-file (filename tar-records)
  (with-open-file (tar filename :direction :output)
    (dolist (record tar-records)
      (write-tar-record record tar)
      (write-tar-data   record tar))
    ;; write the two empty block
    (write-sequence (make-array 1024 :element-type '(unsigned-byte 8) :initial-element 0) tar)))

(defun zero-padded-octal (number)
  (format nil "~6,'0O~C" number #\Null))

(defun write-tar-record (record stream)
  (with-slots (name mode uid gid size mtime chksum typeflag 
	       linkname magic version uname gname devmajor
	       devminor prefix data) record
      (let ((header (make-array 512 :element-type 'character)))
	(setf (subseq header 0   99)  name
	      (subseq header 100 107) (zero-padded-octal mode)
	      (subseq header 108 115) (zero-padded-octal uid)
	      (subseq header 116 123) (zero-padded-octal gid)
	      (subseq header 124 135) (format nil "~12,'0O" size)
	      (subseq header 136 147) (format nil "~12,'0O" mtime)
	      (subseq header 148 155) (zero-padded-octal chksum)
	      (subseq header 156 156) (ecase typeflag
					(:regular "0")
					(:link "1")
					(:symbolic "2")
					(:char-special "3")
					(:block-special "4")
					(:direction "5")
					(:fifo-special "6"))
	      (subseq header 157 256) linkname
	      (subseq header 257 262) (format nil "ustar~C" #\Null)
	      (subseq header 263 264) (format nil "~2,'0O" version)
	      (subseq header 265 296) uname
	      (subseq header 297 328) gname
	      (subseq header 329 336) (zero-padded-octal devmajor)
	      (subseq header 337 344) (zero-padded-octal devminor)
	      (subseq header 345 499) prefix)
	(write-sequence header stream))))

(defun write-tar-data (record stream)
  (with-slots (size data) record
    (loop for offset upfrom 0 by 512 below size
	  do (write-sequence (make-array 512
					 :element-type 'character
					 :displaced-to data
					 :displaced-index-offset offset)
			     stream))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Copyright (c) 2002-2003, Edward Marco Baringer
;;;; All rights reserved. 
;;;; 
;;;; Redistribution and use in source and binary forms, with or without
;;;; modification, are permitted provided that the following conditions are
;;;; met:
;;;; 
;;;;  - Redistributions of source code must retain the above copyright
;;;;    notice, this list of conditions and the following disclaimer.
;;;; 
;;;;  - Redistributions in binary form must reproduce the above copyright
;;;;    notice, this list of conditions and the following disclaimer in the
;;;;    documentation and/or other materials provided with the distribution.
;;;;
;;;;  - Neither the name of Edward Marco Baringer, nor BESE, nor the names
;;;;    of its contributors may be used to endorse or promote products
;;;;    derived from this software without specific prior written permission.
;;;; 
;;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
;;;; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;;;; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
;;;; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
;;;; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;;;; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;;;; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;;;; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;;;; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;;;; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;;;; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
